@@ -1,6 +1,11 @@
 TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsColors=NULL,Imx=NULL,Names=NULL, BmSize=6,...) {
   #author: Tim Schreier, Luis Winckelmann, MCT
   udim <- dim(GeneralizedUmatrix)
+  #Tiled needed for Imx
+  if(!is.null(Imx)){
+    Tiled=TRUE
+  }
+  
   # Error Catching ----
   if (missing(BestMatchingUnits)) {
     BestMatchingUnits = matrix(1, 2, 2)
@@ -99,6 +104,9 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
     Tiled = FALSE
   else
     Tiled=dots[["Tiled"]]
+  if(!is.null(Imx)){
+    Tiled=TRUE
+  }
   
   #axis with labels
   if (is.null(dots[["ShinyBinding"]]))
@@ -125,11 +133,16 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
                        plot,
                        bmu_cols,
                        MarkerSize) {
-    inds <- which(plotCls == class)
+    
+    inds <- which(Cls == class)
+    x = as.numeric(plotbmus[inds, 2])
+    y = as.numeric(plotbmus[inds, 1])
+
     plot <- plotly::add_markers(
-      plot,
-      x = plotbmus[inds, 2],
-      y = plotbmus[inds, 1],
+      plot, x=x, y=y,
+
+
+    
       marker = list(
         size = MarkerSize,
         color = bmu_cols[class],
@@ -138,6 +151,7 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
       ),
       name = paste("Cluster", class)
     )
+
     return(plot)
   }
   
@@ -150,7 +164,63 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
                            ClsColors,
                            MarkerSize,
                            ShinyBinding,
-                           ShinyDimension) {
+                           ShinyDimension,
+                           Imx,Cls) {
+    
+
+    # configure filter, so that every bestmatch stays in
+ 
+    
+    # put Imx on Umatrix and bestmatches if given
+    if(!is.null(Imx)){
+      if(!is.null(plotbmus)){
+        BestMatchesFilter = rep(T,nrow(plotbmus)) # every Bestmatch stays
+        
+      }
+      plotumx[which(Imx == 1)] = 0
+      bigImx = Imx
+      #print(str(BestMatchesFilter))
+      #print(str(plotbmus))
+      for(i in 1:nrow(Imx)){
+        for(j in 1:ncol(Imx)){
+          if(Imx[i,j] == 1){
+            #plotumx[i,j] = NA
+            if(!is.null(plotbmus)){
+              
+              BestMatchesFilter[(plotbmus[,1] == i) & (plotbmus[,2] == j)] = F
+            }
+              
+          }
+        }
+      }
+      #BestMatchesFilter = rep(T,nrow(plotbmus)) # every Bestmatch stays
+      #print(str(BestMatchesFilter))
+      if(!is.null(plotbmus)) plotbmus = plotbmus[BestMatchesFilter,]
+
+      #äprint(str(class))
+      #print(class)
+      #todo, ohne globale variable loesen
+      if(!is.null(Cls)) Cls <<- Cls[BestMatchesFilter]
+      
+      oceanLine = apply(plotumx, 1, function(x) all(x==0))
+      startLine = min(which(!oceanLine),na.rm=T)
+      endLine = length(oceanLine) - min(which(rev(!oceanLine)),na.rm=T) + 1
+      
+      oceanCol = apply(plotumx, 2, function(x) all(x==0))
+      startCol = min(which(!oceanCol),na.rm=T)
+      endCol = length(oceanCol) - min(which(rev(!oceanCol)),na.rm=T) + 1
+   
+      if(!is.null(plotbmus)){
+        plotbmus <- plotbmus - cbind(rep(startLine-1,nrow(plotbmus)),rep(startCol-1,nrow(plotbmus)))
+      }
+      plotumx <- plotumx[startLine:endLine,startCol:endCol]
+      Imx <- Imx[startLine:endLine,startCol:endCol]
+      bigImx <- bigImx[startLine:endLine,startCol:endCol]
+      
+    }
+   
+
+
     ax <- list(
       title = "",
       zeroline = FALSE,
@@ -188,22 +258,31 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
         end = 1,
         size = 1 / 15
       ),
-      # colors = color,
-      colors = colorRamp(colormap[c(rep(3, 6),
-                                    seq(
-                                      from = 4,
+      # colors = colorRamp(colormap[c(rep(3, 6),
+      #                               seq(
+      #                                 from = 4,
+      #                                 to = length(colormap) - 30,
+      #                                 length.out = ceiling(Nrlevels2 + 1) - 7
+      #                               ),
+      #                               length(colormap))]),
+      colors = colorRamp(colormap[c(  1,2,
+                                      seq(
+                                      from = 3,
                                       to = length(colormap) - 30,
-                                      length.out = ceiling(Nrlevels2 + 1) - 7
-                                    ),
+                                      length.out = ceiling(Nrlevels2 + 1)- 4
+                                    ),length(colormap),
                                     length(colormap))]),
+
+
       name = "UMatrix"
       # , showscale = FALSE
     )
-    
-    for (class in unique(plotCls)) {
+
+    for (class in unique(Cls)) {
+      
       plt <- addclass(class, plotbmus, plt, ClsColors, MarkerSize)
     }#end add class
-    
+ 
     plt <- plotly::layout(
       #title <- "Drop plot title here",
       #bgcolor = "rgb(244, 244, 248)",
@@ -215,13 +294,13 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
       #, showlegend = FALSE
     )
 
-    if (isTRUE(ShinyBinding)) {
-      requireNamespace('shiny')
-      shiny::updateSelectInput(session,
-                        "ClsSelect",
-                        label = "Select Class",
-                        choices = unique(Cls))
-    }
+    #if (isTRUE(ShinyBinding)) {
+    #  requireNamespace('shiny')
+    #  shiny::updateSelectInput(session,
+    #                    "ClsSelect",
+    #                    label = "Select Class",
+    #                    choices = unique(Cls))
+    #}
 
     return(plt)
   }
@@ -230,28 +309,45 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
     Cls = rep(1, nrow(BestMatchingUnits))
   
   #Normalizing GeneralizedUmatrix ----
-
+  
  
-  quants2 = quantile(as.vector(GeneralizedUmatrix), c(0.01, 0.5, 0.99))
+  quants = quantile(as.vector(GeneralizedUmatrix), c(0.01, 0.5, 0.99))
+  minU = quants[1]
+  maxU = quants[3]
+  GeneralizedUmatrix = (GeneralizedUmatrix - minU)/(maxU -
+                                                      minU)
+   quants2 = quantile(as.vector(GeneralizedUmatrix), c(0.01, 
+                                                      0.5, 0.99))
   minU2 = quants2[1]
   maxU2 = quants2[3]
   HeightScale = round(maxU2 / (2 * max(minU2, 0.05)), 0)
   stretchFactor = sqrt(nrow(GeneralizedUmatrix) ^ 2 + ncol(GeneralizedUmatrix) ^
                          2) / sqrt(50 ^ 2 + 80 ^ 2)
   Nrlevels2 = 2 * HeightScale * stretchFactor
-  
+  indMax = which(GeneralizedUmatrix > 1, arr.ind = T)
+  indMin = which(GeneralizedUmatrix < 0, arr.ind = T)
+  if (length(indMax) > 0)
+    GeneralizedUmatrix[indMax] = 1
+  if (length(indMin) > 0)
+    GeneralizedUmatrix[indMin] = 0
+
   # GeneralizedUmatrix <- GeneralizedUmatrix * HeightScale * stretchFactor
   if (isTRUE(Tiled)) {
+    
+    tU <- tileGUM(GeneralizedUmatrix,BestMatchingUnits,Cls)
+    GeneralizedUmatrix <- tU$GeneralizedUmatrix
+    BestMatchingUnits <- tU$BestMatchingUnits[,2:3] #no key
+    Cls <- tU$Cls
     qdim <- udim * 2
-    dmx  <- cbind(GeneralizedUmatrix, GeneralizedUmatrix)
-    qmx  <- rbind(dmx, dmx)
-    dbm  <-
-      rbind(BestMatchingUnits,
-            cbind(BestMatchingUnits[, 1], BestMatchingUnits[, 2] + udim[2]))
-    qbm  <- rbind(dbm, cbind(dbm[, 1] + udim[1], dbm[, 2]))
-    plotumx <- qmx
-    plotbmus <- qbm
-    plotCls <- rep(Cls, 4)
+    #dmx  <- cbind(GeneralizedUmatrix, GeneralizedUmatrix)
+    #qmx  <- rbind(dmx, dmx)
+    #dbm  <-
+    #  rbind(BestMatchingUnits,
+    #        cbind(BestMatchingUnits[, 1], BestMatchingUnits[, 2] + udim[2]))
+    #qbm  <- rbind(dbm, cbind(dbm[, 1] + udim[1], dbm[, 2]))
+    plotumx <- tU$GeneralizedUmatrix
+    plotbmus <- tU$BestMatchingUnits[,2:3] #no key
+    plotCls <-  tU$Cls
   } else{
     plotumx <- GeneralizedUmatrix
     plotbmus <- BestMatchingUnits
@@ -260,6 +356,7 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
   }
   
   plotdim <- qdim
+
   plt=PlotlyUmatrix(
     plotdim,
     plotumx,
@@ -270,20 +367,22 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
     ClsColors,
     BmSize,
     ShinyBinding,
-    ShinyDimension
+    ShinyDimension,
+    Imx,
+    Cls
   )
   if(is.null(main))
     plt=plotly::layout(plt,title = "Topographic Map of Generalized U-Matrix")
   else
     plt=plotly::layout(plt,title = main)
   
-  if (isTRUE(ShinyBinding)) {
-    PlotR <- plotly::renderPlotly({
-      plt
-    })
-    return(list(Rendered=PlotR,single=plt))
-  } else{
+  #if (isTRUE(ShinyBinding)) {
+  #  PlotR <- plotly::renderPlotly({
+  #    plt
+  #  })
+  #  return(list(Rendered=PlotR,single=plt))
+  #} else{
     return(plt)
-  }
+  #}
  
 }#end TopviewTopographicMap
